/*******************************************************************************
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL MAXIM INTEGRATED BE LIABLE FOR ANY CLAIM, DAMAGES
 * OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 * Except as contained in this notice, the name of Maxim Integrated
 * Products, Inc. shall not be used except as stated in the Maxim Integrated
 * Products, Inc. Branding Policy.
 *
 * The mere transfer of this software does not imply any licenses
 * of trade secrets, proprietary technology, copyrights, patents,
 * trademarks, maskwork rights, or any other form of intellectual
 * property whatsoever. Maxim Integrated Products, Inc. retains all
 * ownership rights.
 *
 * $Date: 2016-04-08 16:56:13 -0500 (Fri, 08 Apr 2016) $
 * $Revision: 22286 $
 *
 ******************************************************************************/

/***** Includes *****/
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include "mxc_config.h"
#include "clkman.h"
#include "ioman.h"
#include "mxc_sys.h"
#include "nvic_table.h"
#include "tmr.h"
#include "uart.h"
#include "board.h"
#include "gpio.h"
#include "spim.h"
#include "led.h"

/***** Definitions *****/
#define MX25_BAUD           	50000    				// SPI Baud Rate (50kHz)

/***** Definitions *****/
#define UART_TEST               MXC_UART0
#define UART_TEST_IDX           (MXC_UART_GET_IDX(UART_TEST))

/***** Globals *****/
spim_req_t req;
gpio_cfg_t gpio;

/***** Prototypes *****/
void initialise(void);
void spi_write(uint8_t *data);
void ContinuousTimer_handler();
void UART0_IRQHandler(void);
void delay_ms(uint32_t time);							// delay in milliseconds
void set_bit(uint8_t port, uint8_t bit);
void clear_bit(uint8_t port, uint8_t bit);
uint8_t check_input(uint8_t port, uint8_t bit);

/***** Functions *****/
//***************************** Set up Micro ***********************************
void initialise(void)
{
	uint16_t error;
    sys_cfg_spim_t sys_cfg_spi;

    //***************************** Set up Interrupts ******************************
    /*
     * This code is copied from the main file inside
     * C:\Maxim\Firmware\MAX32625\Applications\EvKitExamples\Timer
     *
     */
    // enable timer interrupt - Timer 1
    NVIC_SetVector(TMR1_0_IRQn, ContinuousTimer_handler);
    TMR32_EnableINT(MXC_TMR1);


	/*
	 * This code is copied from the main file inside
	 * C:\Maxim\Firmware\MAX32625\Applications\EvKitExamples\UART
	 *
	 */
    // Setup the UART0 interrupt
    NVIC_ClearPendingIRQ(MXC_UART_GET_IRQ(0));
    NVIC_DisableIRQ(MXC_UART_GET_IRQ(0));
    NVIC_SetPriority(MXC_UART_GET_IRQ(0), 1);
    NVIC_EnableIRQ(MXC_UART_GET_IRQ(0));

    //***************************** Set up Timer 1 *********************************
    /*
     * This code is copied from the main file inside
     * C:\Maxim\Firmware\MAX32625\Applications\EvKitExamples\Timer
     *
     */
    // initialize timer
    tmr_prescale_t prescale = TMR_PRESCALE_DIV_2_12;
    error = TMR_Init(MXC_TMR1, prescale, NULL);

    if(error != E_NO_ERROR)
    {
    	printf("Error in timer setup %d\n", error);
    	delay_ms(10);
    }

    //***************************** GPIO Ports *************************************
    /*
     * This code is copied from the main file inside
     * C:\Maxim\Firmware\MAX32625\Applications\EvKitExamples\LP_GPIO
     *
     */

    // configure GPIO
    // Port 0: Push Pull to 1v8.
    gpio.port = 0;
    gpio.mask = 0xFF;
    gpio.func = GPIO_FUNC_GPIO;
    gpio.pad = GPIO_PAD_NORMAL;							// push pull
    GPIO_Config(&gpio);

    // Port 1: P1.6, P1.7 as push pull outputs
    gpio.port = 1;
    gpio.mask = 0xC0;
    gpio.func = GPIO_FUNC_GPIO;							// configure P1.6, P1.7 as GPIO
    gpio.pad = GPIO_PAD_NORMAL;							// push pull
    GPIO_Config(&gpio);

    // Port 2: Open Drain on P2.4, 2.5, 2.6 - for LEDs
    gpio.port = 2;
    gpio.mask = 0x70;
    gpio.func = GPIO_FUNC_GPIO;							// configure P2.4, P2.5, P2.6 as GPIO
    gpio.pad = GPIO_PAD_OPEN_DRAIN;						// open drain
    GPIO_Config(&gpio);

    // Port 2: switch input on P2.7. Leave others untouched
    gpio.port = 2;
    gpio.mask = 0x80;
    gpio.func = GPIO_FUNC_GPIO;							// configure P2.7 as GPIO
    gpio.pad = GPIO_PAD_INPUT_PULLUP;					// input with weak pullup
    GPIO_Config(&gpio);

    // Port 3 is not used on PICO

    // Port 4: P4.4, P4.5, P4.6, P4.7
    gpio.port = 4;
    gpio.mask = 0xF0;
    gpio.func = GPIO_FUNC_GPIO;							// configure P4.4, P4.5, P4.6, P4.7 as GPIO
    gpio.pad = GPIO_PAD_NORMAL;							// all outputs
    GPIO_Config(&gpio);

    //***************************** Initialise SPIM ********************************
    /*
     * This code is copied from the main file inside
     * C:\Maxim\Firmware\MAX32625\Applications\EvKitExamples\SPIX_MX25
     *
     */
    // Use SPI Master 0
    spim_cfg_t cfg_spi;
    cfg_spi.mode = 3;									// mode 3
    cfg_spi.ssel_pol = 0;
    cfg_spi.baud = MX25_BAUD;

    // SPI 0 port configuration
    // configure I/O manager to enable SPI Master 0 (page 293)
    //                                  			core I/O, ss0, ss1, ss2, ss3, ss4, quad, fast I/O
    sys_cfg_spi.io_cfg = (ioman_cfg_t)IOMAN_SPIM0(			1, 1,    0,   0,   0,   0,    0,     1);
    sys_cfg_spi.clk_scale = CLKMAN_SCALE_DIV_256;		// divide by 256
    SPIM_Init(MXC_SPIM0, &cfg_spi, &sys_cfg_spi);

    //***************************** Initialise UART ********************************
    /*
     * This code is copied from the main file inside
     * C:\Maxim\Firmware\MAX32625\Applications\EvKitExamples\LP_UART\tx
     * PICO uses UART0
     *
    */

    // Initialise the UART
    uart_cfg_t test_uart_cfg;
    test_uart_cfg.parity = UART_PARITY_DISABLE;
    test_uart_cfg.size = UART_DATA_SIZE_8_BITS;
    test_uart_cfg.cts = 0;								// disable RTS & CTS
    test_uart_cfg.rts = 0;
    test_uart_cfg.baud = 1800000;
    test_uart_cfg.extra_stop = 0;

    sys_cfg_uart_t sys_cfg_uart;
    sys_cfg_uart.clk_scale = CLKMAN_SCALE_AUTO;
    sys_cfg_uart.io_cfg = (ioman_cfg_t)IOMAN_UART(0, IOMAN_MAP_A, IOMAN_MAP_A, IOMAN_MAP_A, 1, 1, 1);

    UART_Init(MXC_UART0, &test_uart_cfg, &sys_cfg_uart);

    MXC_UART0->inten |= (MXC_F_UART_INTEN_RX_FIFO_NOT_EMPTY);	// enable RX interrupt
}

/******************************************************************************/
void read_cb(uart_req_t* req, int error)
{

}

// ****************** SPI Transaction ******************************************
void spi_write(uint8_t *data)
{
	/*
	 * This code is copied from mx25.c example program in
	 * C:\Maxim\Firmware\MAX32625\Libraries\Boards\Source
	 *
	 */

    // Send 2 bytes
    req.len     = 2;
    req.tx_data  = data;
    req.rx_data  = NULL;
    req.deass   = 1;									// this line controls CS
    SPIM_Trans(MXC_SPIM0, &req);
}

//***************************** Timer Interrupt Handler ************************
void ContinuousTimer_handler()
{
    TMR32_ClearFlag(MXC_TMR1);
}

// ****************** UART0 Interrupt Handler **********************************
void UART0_IRQHandler(void)
{
	/*
	 * This code is copied from the main file inside
	 * C:\Maxim\Firmware\MAX32625\Applications\EvKitExamples\LP_UART\rx
	 */
	uint8_t data[1];
	mxc_uart_fifo_regs_t *fifo;

	fifo = MXC_UART_GET_FIFO(0);

	MXC_UART0->intfl |= MXC_F_UART_INTFL_TX_DONE;		// clear TX Done flag
	MXC_UART0->intfl |= MXC_F_UART_INTFL_RX_FIFO_NOT_EMPTY;	// clear FIFO not empty flag

	data[0] = fifo->rx;									// dummy read to clear RX FIFO
	// Flash Green LED
	LED_On(1);
	set_bit(4, 4);										// set Port 4, b4
	delay_ms(100);
	LED_Off(1);
	clear_bit(4, 4);									// clear Port 4, b4
}

//***************************** Delay ******************************************
void delay_ms(uint32_t time)							// time in milliseconds
{

	/* This code is copied from the main file inside
	 * C:\Maxim\Firmware\MAX32625\Applications\EvKitExamples\Timer
	 *
	 */

	tmr32_cfg_t cont_cfg;
	uint16_t error;

    cont_cfg.mode = TMR32_MODE_CONTINUOUS;
    cont_cfg.polarity = TMR_POLARITY_INIT_LOW;			//start GPIO low

    // calculate the ticks values for given time
    error = TMR32_TimeToTicks(MXC_TMR1, time, TMR_UNIT_MILLISEC, &(cont_cfg.compareCount));

    if(error != E_NO_ERROR)
    {
    	printf("Error in timer setup %d\n", error);
    	delay_ms(10);
    }

    // configure and start the timer
    TMR32_Config(MXC_TMR1, &cont_cfg);
	TMR32_Start(MXC_TMR1);

    while(MXC_TMR1->intfl == 0)
    {

    }
}

// ****************** Set Port Pin *********************************************
void set_bit(uint8_t port, uint8_t bit)
{
    gpio_cfg_t port_num;
    port_num.port = port;
    port_num.mask = (1<<bit);
	GPIO_OutSet(&port_num);
}

// ****************** Clear Port Pin *******************************************
void clear_bit(uint8_t port, uint8_t bit)
{
    gpio_cfg_t port_num;
    port_num.port = port;
    port_num.mask = (1<<bit);
	GPIO_OutClr(&port_num);
}

// ****************** Check Input Port Pin *************************************
uint8_t check_input(uint8_t port, uint8_t bit)
{
    gpio_cfg_t port_num;
    port_num.port = port;
    port_num.mask = (1<<bit);
    return GPIO_InGet(&port_num);
}

// ****************** MAIN *****************************************************
int main(void)
{
	uint8_t txdata[1];
	uint8_t rxdata[1];
	uint8_t spi_data[2] = {0x55, 0x55};					// SPI data
	txdata[0] = 0xAA;									// UART data

    // Initialise I/O, Timer 1, SPI
    initialise();

    memset(rxdata, 0x0, 1);								// set RX buffer size = 1

    // Setup the asynchronous requests
    uart_req_t read_req;
    read_req.data = rxdata;
    read_req.len = 128;
    read_req.callback = read_cb;

    UART_ReadAsync(MXC_UART1, &read_req);

	// clear screen on terminal program
	printf("\033[2J");							// clear screen
	printf("\033[H");							// cursor home
	delay_ms(10);

    while(1)
    {
        //if(!(check_input(2, 7)))						// uncomment this line to invoke push button
        {
        	// Switch on Red LED for 500ms
        	LED_On(0);
        	delay_ms(500);
        	LED_Off(0);

        	// Switch on Green LED for 500ms
        	LED_On(1);
        	delay_ms(500);
        	LED_Off(1);

        	// Switch on Blue LED for 500ms
        	LED_On(2);
        	delay_ms(500);
        	LED_Off(2);

        	// send 2 bytes via SPIM0
        	spi_write(spi_data);
        	// send 1 byte via UART
        	UART_Write(MXC_UART0, txdata, 1);
        	delay_ms(10);
        	printf("Hello from the PICO\n");
        	delay_ms(10);
        }
    }

}
